import math

def to_seconds(time: str) -> int:
    
    h, m = time.split(':')
    seconds = int(h) * 3600 + int(m) * 60

    return seconds

def to_hours(time: int) -> float:

    h, m = time.split(':')
    hours = int(h) + int(m) / 60

    return hours

def hours_to_string(hours: float) -> str:

    h = int(hours)
    m = hours - h * 60

    return f'{h}:{m}'

def truncate(number, digits) -> float:
    stepper = 10.0 ** digits
    return math.trunc(stepper * number) / stepper


from acme.utils import *


class Employee:

    def __init__(self, name: str):
        self.name = name


class Day:

    _workweek = ['MO', 'TU', 'WE', 'TH', 'FR']

    def __init__(self, abbrev: str):
        self.abbrev = abbrev

    def get_week(self) -> str:
        if self.abbrev in self._workweek:
            return 'workweek'
        else:
            return 'weekend'


class WorkingHours:

    MAX_HOUR = 24

    def __init__(self, start_time: str, end_time: str):
        self.start_time = start_time
        self.end_time = self._check_max_hour(end_time)
        self._to_hours()

    def _check_max_hour(self, end_time: str) -> str:
        if end_time == '00:00':
            end_time = '24:00'

        return end_time

    def _to_hours(self):
        self.start_time = to_hours(self.start_time)
        self.end_time = to_hours(self.end_time)

    def get_total(self) -> float:
        if self.end_time < self.start_time:
            total_hours = self.MAX_HOUR - self.start_time + self.end_time
        else:
            total_hours = self.end_time - self.start_time

        return total_hours


class Shift(WorkingHours):

    def __init__(self, name: str, start_time: str, end_time: str):
        super().__init__(start_time, end_time)
        self.name = name



import unittest
from acme.commands import processing
import contextlib as cl
import io
import re


class TestCommands(unittest.TestCase):

    def test_processing_wrong_arguments(self):
        tests = [
            ['-casdfdsa'],
            ['data.bin'],
            ['-helpasd'],
        ]
        for test in tests:
            out = io.StringIO()
            with cl.redirect_stdout(out):
                processing(test)
            data = out.getvalue()
            ans = '\nInvalid argument.\nFor more information run: acme --help\n'
            # convert item list to string
            if ''.join(test).startswith('-'):
                self.assertEqual(data, ans)
            else:
                ans = '\nCheck the file is in the same directory\nAlso check the correct file extension (.txt)'  + ans
                self.assertEqual(data, ans)

    def test_acme_file(self):
        test = ['acme/data/employees.txt']
        out = io.StringIO()
        with cl.redirect_stdout(out):
            processing(test)
        data = out.getvalue()
        ans = '\nThe amount to pay RENE is: 215.0 USD\nThe amount to pay ASTRID is: 85.0 USD\nThe amount to pay ERICK is: 395.0 USD\n\n\n'
        self.assertEqual(data, ans)
        pass

    def test_acme_instructions(self):
        test = ['--help']
        out = io.StringIO()
        with cl.redirect_stdout(out):
            processing(test)
        data = re.sub(r'[\n\t\s]', '', out.getvalue())
        with open('tests/data/instructions.txt') as f:
            ans = re.sub(r'[\n\t\s]', '', f.read())
        self.assertEqual(data, ans)



from acme.app import run, demo


def check(argv):
    # Check if some command is completed
    completed = 0
    if len(argv) == 1:
        if argv[0] == '-h' or argv[0] == '--help':
            instructions()
            completed = 1
        elif argv[0] == '-d' or argv[0] == '--demo':
            completed = demo()
        else:
            completed = run(argv[0])
    return completed


def processing(argv):
    # processing the command and check argument
    options = ['-d', '--demo', '-h', '--help']
    is_option = argv[0] in options or argv[0].endswith('.txt')
    completed = 0
    res = ''
    if is_option:
        completed = check(argv)
    elif not argv[0].startswith('-'):
        res += '\nCheck the file is in the same directory'
        res += '\nAlso check the correct file extension (.txt)'
    if not completed:
        res += '\nInvalid argument.'
        res += '\nFor more information run: acme --help'
    print(res)


def instructions():
    # Print the instructions

    info = r"""
      ______    ______   __       __  ________
     /      \  /      \ |  \     /  \|        \
    |  $$$$$$\|  $$$$$$\| $$\   /  $$| $$$$$$$$
    | $$__| $$| $$   \$$| $$$\ /  $$$| $$__
    | $$    $$| $$      | $$$$\  $$$$| $$  \
    | $$$$$$$$| $$   __ | $$\$$ $$ $$| $$$$$
    | $$  | $$| $$__/  \| $$ \$$$| $$| $$_____
    | $$  | $$ \$$    $$| $$  \$ | $$| $$     \
     \$$   \$$  \$$$$$$  \$$      \$$ \$$$$$$$$"""
    usage = """
    Usage:
        acme <file>
        
        Ensure the file is a text (.txt) file"""

    options = """
    Options:
        -d  : Create a demo file with default name (data.txt) 
              and execute the app (also --demo)
        -h  : Print the instructions (also --help)"""

    examples = """
    Examples:
        acme data.txt
        acme --demo
        acme -h"""

    help = (
        f'{info}\n'
        f'{usage}\n'
        f'{options}\n'
        f'{examples}'
    )

    print(help)



from acme.entities import Shift

# Working during the day
first_shift = Shift(name='first_shift', start_time='09:01', end_time='18:00')

# Runs from afternoon to midnight
second_shift = Shift(name='second_shift', start_time='18:01', end_time='00:00')

# Shift start at midnight and ends in the morning
third_shift = Shift(name='third_shift', start_time='00:01', end_time='09:00')

# Shifts of the workday
shiftwork = [first_shift, second_shift, third_shift]


import unittest
import os
from unittest.mock import patch

from acme.app import file_path, read_file_lines, calculate_salary, show_salary, calculate_day_cost

# TESTDATA_FILENAME = os.path.join(os.path.dirname(__file__), 'testdata.html')


class TestApp(unittest.TestCase):

    # def setUp(self):
    #    self.testdata = open(TESTDATA_FILENAME).read()

    # def test_find_file(self):
    #     output: str = file_path('data.txt')

    #     self.assertEqual('I:\\Code\\acme\\data.txt', output)

    def test_read_file_lines(self):
        path: str = file_path('acme/data/employees.txt')
        output: str = read_file_lines(path)

        self.assertEqual(
            'RENE=MO10:00-12:00,TU10:00-12:00,TH01:00-03:00,SA14:00-18:00,SU20:00-21:00\n'
            'ASTRID=MO10:00-12:00,TH12:00-14:00,SU20:00-21:00\n'
            'ERICK=MO5:00-11:00,TH8:00-17:00,SU01:00-05:00', output)

    def test_name_in_the_output(self):
        path: str = file_path('acme/data/employees.txt')
        output: str = show_salary('acme/data/employees.txt')

        self.assertRegex(output, 'RENE')
        self.assertRegex(output, 'ASTRID')
        self.assertRegex(output, 'ERICK')

    def test_calculate_salary_amount(self):
        output: float = calculate_salary(
            'MO10:00-12:00,TU10:00-12:00,TH01:00-03:00,SA14:00-18:00,SU20:00-21:00')
        self.assertEqual(215.0, output)
        output: float = calculate_salary(
            'MO10:00-12:00,TH12:00-14:00,SU20:00-21:00')
        self.assertEqual(85.0, output)
        output: float = calculate_salary(
            'MO5:00-11:00,TH8:00-17:00,SU01:00-05:00')
        self.assertEqual(395.0, output)

    def test_calculate_day_cost(self):
        output: float = calculate_day_cost('MO', '10:00-12:00')
        self.assertEqual(30.0, output)
        output: float = calculate_day_cost('SU', '20:00-21:00')
        self.assertEqual(25.0, output)


if __name__ == '__main__':
    unittest.main()



import os
from acme.entities import *
from acme.utils import *
from acme.data.shiftwork import shiftwork
from acme.data.rates import rates
from pathlib import Path

ONE_MINUTE = 1 / 60


def run(param):
    print(show_salary(param))
    return 1


def demo():
    f = open("data.txt", "w+")
    f.write('RENE=MO10:00-12:00,TU10:00-12:00,TH01:00-03:00,SA14:00-18:00,SU20:00-21:00\n'
            'ASTRID=MO10:00-12:00,TH12:00-14:00,SU20:00-21:00')
    f.close()
    print(show_salary('data.txt'))

    return 1


def file_path(relative_path: str) -> str:
    dir: str = os.path.dirname(os.path.abspath(__file__))
    root_dir = Path(dir).parent
    split_path = relative_path.split("/")
    new_path: str = os.path.join(root_dir, *split_path)
    return new_path


def read_file_lines(path: str) -> str:
    with open(path) as file:
        lines: str = ''
        for line in file:
            lines += line
    return lines


def calculate_day_cost(day: str, hours: str) -> float:
    day = Day(day)
    start_time, end_time = hours.split('-')
    working_hours = WorkingHours(start_time, end_time)

    cost: float = 0
    total_hours: float = working_hours.get_total()

    while True:
        for shift in shiftwork:
            if(working_hours.start_time > 24):
                working_hours.start_time -= 24
            if(working_hours.end_time > 24):
                working_hours.end_time -= 24

            if truncate(working_hours.start_time, 1) >= truncate(shift.start_time, 1) and truncate(working_hours.end_time, 1) <= truncate(shift.end_time, 1):
                if working_hours.start_time <= working_hours.end_time:
                    cost += working_hours.get_total() * \
                        rates[shift.name][day.get_week()]
                    working_hours.start_time = working_hours.end_time
                else:
                    cost += (shift.end_time - working_hours.start_time +
                             ONE_MINUTE) * rates[shift.name][day.get_week()]
                    working_hours.start_time = shift.end_time + ONE_MINUTE

            if truncate(working_hours.start_time, 1) >= truncate(shift.start_time, 1) and \
                    truncate(working_hours.start_time, 1) <= truncate(shift.end_time, 1) and \
                    truncate(working_hours.end_time, 1) > truncate(shift.end_time, 1):
                cost += (shift.end_time - working_hours.start_time) * \
                    rates[shift.name][day.get_week()]
                working_hours.start_time = shift.end_time

            if working_hours.get_total() == 0:
                break

        if working_hours.get_total() == 0:
            break

    return cost


def calculate_salary(time_worked: str) -> float:
    days = time_worked.split(',')
    salary: float = 0.0
    for day in days:
        hours = day[2:]
        day = day[:2]
        salary += calculate_day_cost(day, hours)

    return salary


def show_salary(file):
    path: str = file_path(file)
    lines = read_file_lines(path).split('\n')
    output: str = '\n'
    for line in lines:
        name: str = line.split('=')[0]
        amount: float = calculate_salary(line.split('=')[1])
        output += f'The amount to pay {name} is: {amount} USD\n'

    return output


calculate_day_cost('MO', '20:00-21:00')
